#!/usr/bin/env python3

import socket
import sqlite3
import sys


TOKEN_FILE="/space/28c3/workdir/imsiexport"
HLR_FILE="/space/28c3/etc/hlr.sqlite3"
OPENBSC_HOST="127.0.0.1"
OPENBSC_PORT=4242


def load_tokens(filename):
	# Load entire file
	fh = open(filename, 'r')
	data = fh.read()
	fh.close()

	# Hash return
	tokens = {}

	# Process line-by-line
	for l in data.splitlines():
		# Get data
		try:
			token, ext, name = l.split(' ', 2)

			if len(token) != 11:
				raise ValueError('Invalid token')
			if len(ext) != 5:
				raise ValueError('Invalid extension')

		except Exception as e:
			print("[!] Invalid entry in the file ignored: %r (%s)" % (l,e))

		# Add
		tokens[token] = (ext, name)

	# Done
	return tokens


def check_extension(dbc, ext):
	curs = dbc.execute('SELECT imsi FROM Subscriber WHERE extension = ?', [ext])
	row = curs.fetchone()
	if row is None:
		return None
	imsi = '%d' % row[0]
	curs.close()
	return imsi


def free_extension(dbc, ext):
	curs = dbc.execute('UPDATE Subscriber SET extension = (id-1+900000), name = ? WHERE extension = ?', ['Unknown', ext])
	curs.close()


def assign_extension(dbc, ext, name, imsi):
	curs = dbc.execute('UPDATE Subscriber SET extension = ?, name = ? WHERE imsi = ?', [ext, name, imsi])
	curs.close()


def bust_openbsc_cache(host, port, imsi):
	# Connect to openbsc
	s = socket.socket()
	s.connect( (host, port) )

	# Issue commands
	s.send(("enable\n").encode('ascii'))
	s.send(("subscriber imsi %s update\n" % (imsi,)).encode('ascii'))

	# Readoff stuff
	s.recv(2048)

	# Done
	s.close()


def main():

	# Args
	if len(sys.argv) < 7:
		print("Not enough parameters %r" % (sys.argv,))
		sys.exit(-1)

	token = sys.argv[1]
	imsi  = sys.argv[6]

	# Pre check token
	if len(token) != 11 or token[0] != '9':
		print("Invalid token format %r\n" % (sys.argv,))
		sys.exit(-2)

	# Load token list
	tokens_ht = load_tokens(TOKEN_FILE)

	# Check validity & get extension
	if token not in tokens_ht:
		print("Given token %s not registered at eventphone" % (token,))
		sys.exit(-2)

	extension, name = tokens_ht[token]

	# Open database
	db_conn = sqlite3.connect(HLR_FILE)

	# Check extension assignement
	imsi_assigned = check_extension(db_conn, extension)

	# Anything to do ?
	if imsi_assigned != imsi:
		# IMSI to bust later on
		to_bust = []

		# Free if there was one
		if imsi_assigned:
			free_extension(db_conn, extension)
			to_bust.append(imsi_assigned)

		# Assign new one
		assign_extension(db_conn, extension, name, imsi)
		to_bust.append(imsi)

		# Save (so that openbsc can read)
		db_conn.commit()

		# Clear openbsc cache
		for tb in to_bust:
			try:
				bust_openbsc_cache(OPENBSC_HOST, OPENBSC_PORT, tb)
			except Exception as e:
				print("Failure to bust openbsc cache for imsi %s (%s)" % (tb, e))

	# Close database
	db_conn.close()


if __name__ == '__main__':
	main()

